/////////////////////////////////////////////////////////////////////////////
// hip.cpp: application implementation file



#include "stdafx.h"
#include "resource.h"

#include "hip.h"

#include <time.h>	// beep, sleep & rand
#include <conio.h>	// beep
#include <stdio.h>	// rand
#include <stdlib.h>	// rand


// Status Bar Indicators
static UINT BASED_CODE indicators[] =
{
	ID_SEPARATOR,			// status line indicator
	ID_INDICATOR_COMPUTER_MOVE,
	ID_INDICATOR_HUMAN_MOVE,
};


/////////////////////////////////////////////////////////////////////////////
// CSquare
//
void CSquare::Traslate(int dx, int dy)
{
	m_1st.x += dx;
	m_1st.y += dy;
	m_2nd.x += dx;
	m_2nd.y += dy;
	m_3rd.x += dx;
	m_3rd.y += dy;
	m_4th.x += dx;
	m_4th.y += dy;
}
	
BOOL CSquare::operator==(CSquare square) 
{ 
	return((m_1st==square.m_1st)
			&& (m_2nd==square.m_2nd)
			&& (m_3rd==square.m_3rd)
			&& (m_4th==square.m_4th));   
}
	
CSquare& CSquare::operator=(CSquare square)     
{
	m_1st = square.m_1st;
	m_2nd = square.m_2nd;
	m_3rd = square.m_3rd;
	m_4th = square.m_4th;   
	return(*this);
}





/////////////////////////////////////////////////////////////////////////////
// CMainWindow constructor:
// Create the window with the appropriate style, size, menu, etc.
//
CMainWindow::CMainWindow()
{
	// Main Window Style
	DWORD wndStyle = WS_VISIBLE|WS_OVERLAPPED|
					 WS_CAPTION|WS_SYSMENU|WS_MINIMIZEBOX;

	// Calculate Window Size
	CRect wndRect;
	CClientDC dc(this);

	int wndWidth = 2*BORDER_WIDTH + HORZ_TILES*BUTTON_WIDTH;
	int wndHeight = 2*BORDER_WIDTH + VERT_TILES*BUTTON_WIDTH;
	wndRect.left = (dc.GetDeviceCaps(HORZRES)-wndWidth)/2;
	wndRect.top = (dc.GetDeviceCaps(VERTRES)-wndHeight)/2;
	wndRect.right = wndRect.left + wndWidth;
	wndRect.bottom = wndRect.top + wndHeight;

	::AdjustWindowRect(&wndRect,wndStyle,TRUE);

	Create(NULL,
			"Hip! for Windows",
			wndStyle,
			wndRect,
			NULL,
			"MainMenu");
                        
	// Load Accelerator Table
	LoadAccelTable("MAINACCELTABLE");
	                        
	// Create Status Bar
	if ((!m_statusBar.Create(this))
		|| (!m_statusBar.SetIndicators(indicators,
		  sizeof(indicators)/sizeof(UINT)))) 
	{
		AfxMessageBox("Can't create the Status Bar",MB_ICONINFORMATION);
		return;
	}
	
	// Adjust Window Rect According to Status Bar Size
	RecalcLayout();
	CRect statusBarRect;
	m_statusBar.GetWindowRect(statusBarRect);
	GetWindowRect(wndRect);

	wndRect.bottom += statusBarRect.Height();
	MoveWindow(wndRect,FALSE);
	
	// Set Status Bar Properties
	m_statusBar.SetPaneInfo(0,ID_SEPARATOR,SBPS_STRETCH,0);	
	m_statusBar.SetPaneInfo(1,ID_INDICATOR_COMPUTER_MOVE,SBPS_NORMAL,wndRect.Width()/6);	
	m_statusBar.SetPaneInfo(2,ID_INDICATOR_HUMAN_MOVE,SBPS_NORMAL,wndRect.Width()/6);	

	LOGFONT fontInfo;
	CFont* pFont = m_statusBar.GetFont();
    pFont->GetObject(sizeof(LOGFONT),&fontInfo);
    
	// Modify Window Font 
    fontInfo.lfHeight = -STATUS_BAR_FONT_HEIGHT;
    _fstrcpy(fontInfo.lfFaceName,STATUS_BAR_FONT_NAME);
    
    pFont->DeleteObject();
    pFont->CreateFontIndirect(&fontInfo);
	
	// Load Strings
	m_winStr.LoadString(IDS_WIN);
	m_lossStr.LoadString(IDS_LOSS);
	m_drawStr.LoadString(IDS_DRAW);

	m_waitingStr.LoadString(IDS_WAITING);
	m_thinkingStr.LoadString(IDS_THINKING);

	m_humanStr.LoadString(ID_INDICATOR_HUMAN_MOVE);
	m_computerStr.LoadString(ID_INDICATOR_COMPUTER_MOVE);
	
	// Calculate Squares Placement
	InitSquares();
	
	// Set RAND Seed
	srand((unsigned)time(NULL));

	m_started = FALSE;
}

void CMainWindow::OnDestroy()
{
	// Destroy the buttons           
	BeginWaitCursor();
	
	for (int i=0; i<=VERT_TILES+1; i++)
		for (int j=0; j<=HORZ_TILES+1; j++)
	{		
		m_gameArea[i][j]->DestroyWindow();
		delete m_gameArea[i][j];	
	}
	
	EndWaitCursor();

	CFrameWnd ::OnDestroy();
}


/////////////////////////////////////////////////////////////////////////////
// Data Initialization
void CMainWindow::InitSquares()
{
	m_totSquares = 0;
	CSquare sample;      
	
	int xDispl = 1;
	int yDispl = 0;
	
	do {
		// First Normalized Square
		sample.m_1st = CPoint(0,0);
		sample.m_2nd.x = sample.m_1st.x + xDispl;
		sample.m_2nd.y = sample.m_1st.y + yDispl;
		sample.m_3rd.x = sample.m_2nd.x - yDispl;
		sample.m_3rd.y = sample.m_2nd.y + xDispl;
		sample.m_4th.x = sample.m_3rd.x - xDispl;
		sample.m_4th.y = sample.m_3rd.y - yDispl;
		
		// Shift the Square into the Game Board
		if (sample.m_4th.x<0)
		{
			sample.m_1st.x += -sample.m_4th.x;
			sample.m_2nd.x += -sample.m_4th.x;
			sample.m_3rd.x += -sample.m_4th.x;
			sample.m_4th.x += -sample.m_4th.x;
		}
		
		if ((sample.m_2nd.x>=HORZ_TILES)
			|| (sample.m_3rd.y>=VERT_TILES))
		{
			// This Square is Too Big!
			// Try with the Next Family...
			yDispl++;
			xDispl = 1;
		}                         
		else
		{
			BOOL done = FALSE;
			CSquare pattern = sample;
			
			do {
				// Good! Store it...
				m_squares[m_totSquares] = pattern;
				m_totSquares++;

				if (m_totSquares>=MAX_SQUARES)
				{
					AfxMessageBox("Square Array Full.",MB_ICONSTOP);
					PostMessage(WM_CLOSE);
					break;
				}             
				
            	pattern.Traslate(1,0);
            	if (pattern.m_2nd.x>=HORZ_TILES)
            	{
            		// Next Row...
            		pattern.Traslate(-pattern.m_4th.x,1);
            		done = (pattern.m_3rd.y>=VERT_TILES);
            	}
			
			} while (!done);

			xDispl++;
		}
					
	} while (yDispl<VERT_TILES); 
}

void CMainWindow::CreateFrame()
{
	// Create the Border Buttons
	CRect rect;
	
	// Top-Left Corner
	rect.top = 0;
	rect.left = 0;
	rect.bottom = BORDER_WIDTH;
	rect.right = BORDER_WIDTH;
	m_gameArea[0][0] = new CBitmapButton;
	m_gameArea[0][0]->Create("caption",
		 					WS_CHILD|WS_VISIBLE|BS_OWNERDRAW,
		 					rect,
		 					this,
		 					ID_USER_SPACE);
    
	// Top Border
	rect.top = 0;
	rect.left = BORDER_WIDTH;
	rect.bottom = BORDER_WIDTH;
	rect.right = BORDER_WIDTH + BUTTON_WIDTH;
	for (int i=0; i<HORZ_TILES; i++)
	{
		m_gameArea[0][i+1] = new CBitmapButton;
		m_gameArea[0][i+1]->Create("caption",
			 					WS_CHILD|WS_VISIBLE|BS_OWNERDRAW,
			 					rect,
		 						this,
		 						ID_USER_SPACE);
		rect.left += BUTTON_WIDTH;
		rect.right += BUTTON_WIDTH;
	}
	
	// Top-Right Corner
	rect.top = 0;
	rect.left = BORDER_WIDTH + HORZ_TILES*BUTTON_WIDTH;
	rect.bottom = BORDER_WIDTH;
	rect.right = rect.left + BORDER_WIDTH;
	m_gameArea[0][HORZ_TILES+1] = new CBitmapButton;
	m_gameArea[0][HORZ_TILES+1]->Create("caption",
		 					WS_CHILD|WS_VISIBLE|BS_OWNERDRAW,
		 					rect,
		 					this,
		 					ID_USER_SPACE);

	// Right Border
	rect.top = BORDER_WIDTH;
	rect.left = BORDER_WIDTH + HORZ_TILES*BUTTON_WIDTH;
	rect.bottom = rect.top + BUTTON_WIDTH;
	rect.right = rect.left + BORDER_WIDTH;
	for (i=0; i<VERT_TILES; i++)
	{
		m_gameArea[i+1][HORZ_TILES+1] = new CBitmapButton;
		m_gameArea[i+1][HORZ_TILES+1]->Create("caption",
			 					WS_CHILD|WS_VISIBLE|BS_OWNERDRAW,
			 					rect,
		 						this,
		 						ID_USER_SPACE);
		rect.top += BUTTON_WIDTH;
		rect.bottom += BUTTON_WIDTH;
	}
	
	// Bottom-Right Corner
	rect.top = BORDER_WIDTH + VERT_TILES*BUTTON_WIDTH;
	rect.left = BORDER_WIDTH + HORZ_TILES*BUTTON_WIDTH;
	rect.bottom = 2*BORDER_WIDTH + VERT_TILES*BUTTON_WIDTH;
	rect.right = 2*BORDER_WIDTH + HORZ_TILES*BUTTON_WIDTH;
	m_gameArea[VERT_TILES+1][HORZ_TILES+1] = new CBitmapButton;
	m_gameArea[VERT_TILES+1][HORZ_TILES+1]->Create("caption",
		 					WS_CHILD|WS_VISIBLE|BS_OWNERDRAW,
		 					rect,
		 					this,
		 					ID_USER_SPACE);

	// Bottom Border
	rect.top = BORDER_WIDTH + VERT_TILES*BUTTON_WIDTH;
	rect.left = BORDER_WIDTH;
	rect.bottom = 2*BORDER_WIDTH + VERT_TILES*BUTTON_WIDTH;
	rect.right = rect.left + BUTTON_WIDTH;
	for (i=0; i<HORZ_TILES; i++)
	{
		m_gameArea[VERT_TILES+1][i+1] = new CBitmapButton;
		m_gameArea[VERT_TILES+1][i+1]->Create("caption",
			 					WS_CHILD|WS_VISIBLE|BS_OWNERDRAW,
			 					rect,
		 						this,
		 						ID_USER_SPACE);
		rect.left += BUTTON_WIDTH;
		rect.right += BUTTON_WIDTH;
	}
	
	// Bottom-Left Corner
	rect.top = BORDER_WIDTH + VERT_TILES*BUTTON_WIDTH;
	rect.left = 0;
	rect.bottom = 2*BORDER_WIDTH + VERT_TILES*BUTTON_WIDTH;
	rect.right = BORDER_WIDTH;
	m_gameArea[VERT_TILES+1][0] = new CBitmapButton;
	m_gameArea[VERT_TILES+1][0]->Create("caption",
		 					WS_CHILD|WS_VISIBLE|BS_OWNERDRAW,
		 					rect,
		 					this,
		 					ID_USER_SPACE);

	// Left Border
	rect.top = BORDER_WIDTH;
	rect.left = 0;
	rect.bottom = rect.top + BUTTON_WIDTH;
	rect.right = rect.left + BORDER_WIDTH;
	for (i=0; i<VERT_TILES; i++)
	{
		m_gameArea[i+1][0] = new CBitmapButton;
		m_gameArea[i+1][0]->Create("caption",
			 					WS_CHILD|WS_VISIBLE|BS_OWNERDRAW,
			 					rect,
		 						this,
		 						ID_USER_SPACE);
		rect.top += BUTTON_WIDTH;
		rect.bottom += BUTTON_WIDTH;
	}
}

void CMainWindow::CreateTiles()
{
	// Create all the Tiles Bitmap Buttons
	CRect rect;
	BeginWaitCursor();
	
	for (int i=1; i<=VERT_TILES; i++)
		for (int j=1; j<=HORZ_TILES; j++)
		{
			rect.top = BORDER_WIDTH + (i-1)*BUTTON_WIDTH;
			rect.left = BORDER_WIDTH + (j-1)*BUTTON_WIDTH;
			rect.bottom = rect.top + BUTTON_WIDTH;
			rect.right = rect.left + BUTTON_WIDTH;
			
			m_gameArea[i][j] = new CBitmapButton;
			m_gameArea[i][j]->Create("caption",
		 							WS_CHILD|WS_VISIBLE|BS_OWNERDRAW,
		 							rect,
		 							this,
		 							ID_USER_SPACE + (i-1)*HORZ_TILES + j);
		} 					  
	
	EndWaitCursor();
}

void CMainWindow::StartGame()
{
	// Draw the Board
	CreateFrame();
	CreateTiles();
	DrawFrame();	       

	// Start a New Game
	m_started = TRUE;
	OnGameNew();
}


/////////////////////////////////////////////////////////////////////////////
// Sound Generator
void CMainWindow::Beep(int frequency,int duration)
{
    int control;

    // If frequency is 0, Beep doesn't try to make a sound. It
    // just sleeps for the duration requested

    if (frequency)
    {
        // Prepare timer by sending 10111100 to port 43
        _outp(0x43,0xb6);

        // Divide input frequency by timer ticks per second and
        // write (byte by byte) to timer

        frequency = (unsigned)(1193180L / frequency);
        _outp(0x42,(char)frequency);
        _outp(0x42,(char)(frequency>>8));

        // Save speaker control byte
        control = inp(0x61);

        // Turn on the speaker (with bits 0 and 1)
        _outp(0x61,control|0x3);
    }

	Sleep((clock_t)duration);

    // Turn speaker back on if necessary
    if (frequency)
        _outp(0x61, control);
}

void CMainWindow::Sleep(clock_t wait)
{
    clock_t goal;

    goal = wait + clock();
    while(goal>clock())
        ;
}


/////////////////////////////////////////////////////////////////////////////
// Window Painting
void CMainWindow::DrawTiles()
{
	for (int i=0; i<VERT_TILES; i++)
		for (int j=0; j<HORZ_TILES; j++)
		{			
			if (m_gameTile[i][j]==EMPTY)
				m_gameArea[i+1][j+1]->LoadBitmaps("IDB_EMPTY");

			if (m_gameTile[i][j]==HUMAN)
				m_gameArea[i+1][j+1]->LoadBitmaps("IDB_HUMAN");

			if (m_gameTile[i][j]==COMPUTER)
				m_gameArea[i+1][j+1]->LoadBitmaps("IDB_COMPUTER");
			
			m_gameArea[i+1][j+1]->RedrawWindow();
		}
}

void CMainWindow::DrawFrame()
{
	// Top-Left Corner
	m_gameArea[0][0]->LoadBitmaps("IDB_BORDER_TL");
	m_gameArea[0][0]->RedrawWindow();
    
	// Top Border
	for (int i=0; i<HORZ_TILES; i++)
	{
		m_gameArea[0][i+1]->LoadBitmaps("IDB_BORDER_HORZ");
		m_gameArea[0][i+1]->RedrawWindow();
	}
	
	// Top-Right Corner
	m_gameArea[0][HORZ_TILES+1]->LoadBitmaps("IDB_BORDER_TR");
	m_gameArea[0][HORZ_TILES+1]->RedrawWindow();

	// Right Border
	for (i=0; i<VERT_TILES; i++)
	{
		m_gameArea[i+1][HORZ_TILES+1]->LoadBitmaps("IDB_BORDER_VERT");
		m_gameArea[i+1][HORZ_TILES+1]->RedrawWindow();
	}
	
	// Bottom-Right Corner
	m_gameArea[VERT_TILES+1][HORZ_TILES+1]->LoadBitmaps("IDB_BORDER_BR");
	m_gameArea[VERT_TILES+1][HORZ_TILES+1]->RedrawWindow();

	// Bottom Border
	for (i=0; i<HORZ_TILES; i++)
	{
		m_gameArea[VERT_TILES+1][i+1]->LoadBitmaps("IDB_BORDER_HORZ");
		m_gameArea[VERT_TILES+1][i+1]->RedrawWindow();
	}
	
	// Bottom-Left Corner
	m_gameArea[VERT_TILES+1][0]->LoadBitmaps("IDB_BORDER_BL");
	m_gameArea[VERT_TILES+1][0]->RedrawWindow();

	// Left Border
	for (i=0; i<VERT_TILES; i++)
	{
		m_gameArea[i+1][0]->LoadBitmaps("IDB_BORDER_VERT");
		m_gameArea[i+1][0]->RedrawWindow();
    }
}


/////////////////////////////////////////////////////////////////////////////
//  GAME Menu Management
void CMainWindow::OnGameNew()
{
	// Set Initial Tile Configuration
	for (int i=0; i<VERT_TILES; i++)
		for (int j=0; j<HORZ_TILES; j++)     

			m_gameTile[i][j] = EMPTY;
			
	// Set Started Flag
	m_started = TRUE;

	// Draw Game Board
	DrawTiles();

	// Who Starts?
	if (rand()%2)
	{
		// Human Starting
		CStartDlg dlg;
		dlg.m_who = "You";
		dlg.Create();
		
		// Wait
		DWORD tm = GetTickCount() + 1000;
		while (tm>GetTickCount());
		
		dlg.DestroyWindow();
	}
	else
	{	
		// Computer Starting
		CStartDlg dlg;
		dlg.m_who = "PC";
		dlg.Create();
		
		// Wait
		DWORD tm = GetTickCount() + 1000;
		while (tm>GetTickCount());
		
		dlg.DestroyWindow();
		PlaceComputerTile();
	}
}

void CMainWindow::OnGameSoundon()
{
	m_sound = !m_sound;
}

void CMainWindow::OnUpdateGameSoundon(CCmdUI* pCmdUI)
{
	pCmdUI->SetCheck(m_sound);
}

void CMainWindow::OnGameLevelLow()
{
	m_skill = LOW_SKILL;
}

void CMainWindow::OnUpdateGameLevelLow(CCmdUI* pCmdUI)
{
	pCmdUI->SetCheck(m_skill==LOW_SKILL);
}

void CMainWindow::OnGameLevelMedium()
{
	m_skill = MED_SKILL;
}

void CMainWindow::OnUpdateGameLevelMedium(CCmdUI* pCmdUI)
{
	pCmdUI->SetCheck(m_skill==MED_SKILL);
}

void CMainWindow::OnGameLevelHigh()
{
	m_skill = HIGH_SKILL;
}

void CMainWindow::OnUpdateGameLevelHigh(CCmdUI* pCmdUI)
{
	pCmdUI->SetCheck(m_skill==HIGH_SKILL);
}

void CMainWindow::OnGameExit()
{
	// Save .INI Data
	AfxGetApp()->WriteProfileInt(OPTIONS_SECTION,
								 SOUND_ENTRY,
								 m_sound);
	AfxGetApp()->WriteProfileInt(OPTIONS_SECTION,
								 PC_SKILL_ENTRY,
								 m_skill);
	// Close Window
	PostMessage(WM_CLOSE);
}

void CMainWindow::OnGameRules()
{
	CDialog dlg("RULESDLG",this);
	dlg.DoModal();
}

void CMainWindow::OnGameAbout()
{
	CDialog about("AboutBox",this);
	about.DoModal();
}


/////////////////////////////////////////////////////////////////////////////
// Ending Game Manager
BOOL CMainWindow::CheckForSquares(int* pWhoSquared /*=NULL*/)
{
	int count = 0;
	CPoint p1,p2,p3,p4;
	BOOL squareFound = FALSE;            
		
	while ((count<m_totSquares) && (!squareFound))
	{
		p1 = m_squares[count].m_1st;
		p2 = m_squares[count].m_2nd;
		p3 = m_squares[count].m_3rd;
		p4 = m_squares[count].m_4th;
		
		// Found a Square Made Up of Tiles of the Same Kind?
		squareFound = ((m_gameTile[p1.y][p1.x]!=EMPTY)
						&& (m_gameTile[p1.y][p1.x]==m_gameTile[p2.y][p2.x])
						&& (m_gameTile[p2.y][p2.x]==m_gameTile[p3.y][p3.x])
						&& (m_gameTile[p3.y][p3.x]==m_gameTile[p4.y][p4.x]));
		count++;
	}

	// Light the Square's Tiles          
	if (squareFound)
	{
		CString res;
		if (m_gameTile[p1.y][p1.x]==HUMAN)
			res = "IDB_HUMAN_OFF";
		else	
			res = "IDB_COMPUTER_OFF";
		
		m_gameArea[p1.y + 1][p1.x + 1]->LoadBitmaps(res);
		m_gameArea[p2.y + 1][p2.x + 1]->LoadBitmaps(res);
		m_gameArea[p3.y + 1][p3.x + 1]->LoadBitmaps(res);
		m_gameArea[p4.y + 1][p4.x + 1]->LoadBitmaps(res);
		m_gameArea[p1.y + 1][p1.x + 1]->RedrawWindow();
		m_gameArea[p2.y + 1][p2.x + 1]->RedrawWindow();
		m_gameArea[p3.y + 1][p3.x + 1]->RedrawWindow();
		m_gameArea[p4.y + 1][p4.x + 1]->RedrawWindow();
	
		// Who Made the Square?
		if (pWhoSquared!=NULL)
			*pWhoSquared = m_gameTile[p1.y][p1.x];
	}
	else
	{
		// Are There Other Empty Tiles?
		BOOL foundEmptyTile = FALSE;
		
		for (int row=0; ((row<VERT_TILES) && (!foundEmptyTile)); row++)
			for (int col=0; ((col<HORZ_TILES) && (!foundEmptyTile)); col++)
			{
    			if (m_gameTile[row][col]==EMPTY)
    				foundEmptyTile = TRUE;
    		}
		
		if (foundEmptyTile)
			*pWhoSquared = EMPTY;
		
	}		
	
	return squareFound;
	
}

void CMainWindow::EndGame(int looserTile)
{
	if (looserTile==HUMAN)
		m_statusBar.SetWindowText(m_winStr);

	if (looserTile==EMPTY)
		m_statusBar.SetWindowText(m_drawStr);

	if (looserTile==COMPUTER)
		m_statusBar.SetWindowText(m_lossStr);

	if (m_sound)
	{
		Beep(880,100);	// A
		Beep(1046,500);	// C
	}

	m_started = FALSE;
}


/////////////////////////////////////////////////////////////////////////////
// Tiles Placement
BOOL CMainWindow::OnCommand(WPARAM wParam, LPARAM lParam)
{
	int looserTile;
	
	if ((m_started)
		&& (wParam>=ID_USER_SPACE)
		&& (wParam<=ID_USER_SPACE + VERT_TILES*HORZ_TILES))
	{
		wParam -= (ID_USER_SPACE + 1);
		int row = (wParam / HORZ_TILES);
		int col = (wParam % HORZ_TILES);
	
		if (m_gameTile[row][col]==EMPTY)     
		{
			PlaceHumanTile(row,col);
			
			// Has Human Create a Square?
			if (!CheckForSquares(&looserTile))
			{
				// NO! Game Must Proceed...
				PlaceComputerTile();        

				if (CheckForSquares(&looserTile))
					EndGame(looserTile);
			}
			else
				// YES! Human Looses.
				EndGame(looserTile);
		}
		else
			AfxMessageBox("Square isn't free!",MB_ICONEXCLAMATION);
			
		return TRUE;
	}	
	else
		return CFrameWnd::OnCommand(wParam,lParam);
}

void CMainWindow::PlaceHumanTile(int row, int col)
{
	m_gameTile[row][col] = HUMAN;
	
	m_gameArea[row+1][col+1]->LoadBitmaps("IDB_HUMAN");
	m_gameArea[row+1][col+1]->RedrawWindow();

	// Write Out Current Tile
	CString yourMove = m_humanStr;
	yourMove += ('A' + (char)col);
	yourMove += ('1' + (char)row);
	m_statusBar.SetPaneText(2,yourMove);
	m_statusBar.RedrawWindow();
	
	if (m_sound)
		MessageBeep(-1);
}

void CMainWindow::PlaceComputerTile()
{
	BeginWaitCursor();
	m_statusBar.SetWindowText(m_thinkingStr);
	m_statusBar.RedrawWindow();
	
	// Sleep...
	DWORD tm = GetTickCount() + THINKING_TIMEOUT;
	while (tm>GetTickCount());
	
	// Find Out Free Tiles
	int row,col,count = 0;
	CPoint freeTiles[VERT_TILES*HORZ_TILES];
	
	for (row=0; row<VERT_TILES; row++)
		for (col=0; col<HORZ_TILES; col++)
		
    		if (m_gameTile[row][col]==EMPTY)
    		{	
    			freeTiles[count].x = col;
    			freeTiles[count].y = row;
    			count++;
            }           
            
	// Rearrange Empty Tiles Randomly
	int i;
	CPoint auxPt;
	for (i=0; i<count; i++)
	{
		row = rand() % count;
		col = rand() % count;
		
		// Swap the Two Points
		auxPt = freeTiles[row];
		freeTiles[row] = freeTiles[col];
		freeTiles[col] = auxPt;
	}
	
	// Select the Position
	i = 0;
	CPoint p1,p2,p3,p4;
	BOOL killer, found = FALSE;

	while ((i<count) && (!found))
	{
		// Skip the Anti-Square Control?
		if ((rand()%100)<m_skill)
			found = TRUE;
		else
		{                 
			int j = 0;

			// Is This a Safe Tile?
			do			
			{
				killer = FALSE;
				p1 = m_squares[j].m_1st;
				p2 = m_squares[j].m_2nd;
				p3 = m_squares[j].m_3rd;
				p4 = m_squares[j].m_4th;
			
				if (freeTiles[i]==p1)
					
					killer = ((m_gameTile[p2.y][p2.x]==COMPUTER)
								&& (m_gameTile[p3.y][p3.x]==COMPUTER)
								&& (m_gameTile[p4.y][p4.x]==COMPUTER));
					
				if (freeTiles[i]==p2)
					
					killer = ((m_gameTile[p1.y][p1.x]==COMPUTER)
								&& (m_gameTile[p3.y][p3.x]==COMPUTER)
								&& (m_gameTile[p4.y][p4.x]==COMPUTER));
					
				if (freeTiles[i]==p3)
					
					killer = ((m_gameTile[p1.y][p1.x]==COMPUTER)
								&& (m_gameTile[p2.y][p2.x]==COMPUTER)
								&& (m_gameTile[p4.y][p4.x]==COMPUTER));
					
				if (freeTiles[i]==p4)
					
					killer = ((m_gameTile[p1.y][p1.x]==COMPUTER)
								&& (m_gameTile[p2.y][p2.x]==COMPUTER)
								&& (m_gameTile[p3.y][p3.x]==COMPUTER));
									
				j++;
			
			} while ((j<m_totSquares) && (!killer));
			
			found = !killer;	// OK only if <killer> is FALSE
		}		
		
		i++;
	}
	    
	// Make the Move...
	row = freeTiles[i-1].y;
	col = freeTiles[i-1].x;
	            
	m_gameTile[row][col] = COMPUTER;

	m_gameArea[row+1][col+1]->LoadBitmaps("IDB_COMPUTER");
	m_gameArea[row+1][col+1]->RedrawWindow();

	// Write Out PC Move
	CString myMove = m_computerStr;
	myMove += ('A' + (char)col);
	myMove += ('1' + (char)row);
	m_statusBar.SetPaneText(1,myMove);
	m_statusBar.RedrawWindow();
	
	if (m_sound)
		MessageBeep(-1);
		
	EndWaitCursor();
	m_statusBar.SetWindowText(m_waitingStr);
}    


/////////////////////////////////////////////////////////////////////////////
// Status Bar Pane Enabling
void CMainWindow::UpdateHumanPane(CCmdUI* pCmdUI)
{
	pCmdUI->Enable(TRUE);
}

void CMainWindow::UpdateComputerPane(CCmdUI* pCmdUI)
{
	pCmdUI->Enable(TRUE);
}

            
// CMainWindow message map:
// Associate messages with member functions.

BEGIN_MESSAGE_MAP( CMainWindow, CFrameWnd )
	//{{AFX_MSG_MAP( CMainWindow )
	ON_COMMAND(ID_GAME_NEW, OnGameNew)
	ON_COMMAND(ID_GAME_EXIT, OnGameExit)
	ON_COMMAND(ID_GAME_ABOUT, OnGameAbout)
	ON_COMMAND(ID_GAME_RULES, OnGameRules)
	ON_COMMAND(ID_GAME_SOUNDON, OnGameSoundon)
	ON_UPDATE_COMMAND_UI(ID_GAME_SOUNDON, OnUpdateGameSoundon)
	ON_WM_DESTROY()
	ON_COMMAND(ID_GAME_LEVEL_HIGH, OnGameLevelHigh)
	ON_UPDATE_COMMAND_UI(ID_GAME_LEVEL_HIGH, OnUpdateGameLevelHigh)
	ON_COMMAND(ID_GAME_LEVEL_LOW, OnGameLevelLow)
	ON_UPDATE_COMMAND_UI(ID_GAME_LEVEL_LOW, OnUpdateGameLevelLow)
	ON_COMMAND(ID_GAME_LEVEL_MEDIUM, OnGameLevelMedium)
	ON_UPDATE_COMMAND_UI(ID_GAME_LEVEL_MEDIUM, OnUpdateGameLevelMedium)
	//}}AFX_MSG_MAP
	ON_UPDATE_COMMAND_UI(ID_INDICATOR_HUMAN_MOVE, UpdateHumanPane)
	ON_UPDATE_COMMAND_UI(ID_INDICATOR_COMPUTER_MOVE, UpdateComputerPane)
END_MESSAGE_MAP()





/////////////////////////////////////////////////////////////////////////////
// CStartDlg dialog

CStartDlg::CStartDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CStartDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CStartDlg)
		// NOTE: the ClassWizard will add member initialization here
	//}}AFX_DATA_INIT
}

BOOL CStartDlg::Create(CWnd* pParent /*=NULL*/)
{
	return CDialog::Create(CStartDlg::IDD, pParent);
}

void CStartDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CStartDlg)
		// NOTE: the ClassWizard will add DDX and DDV calls here
	//}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(CStartDlg, CDialog)
	//{{AFX_MSG_MAP(CStartDlg)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CStartDlg message handlers

BOOL CStartDlg::OnInitDialog()
{
	CDialog::OnInitDialog();
	
	CString txt;
	GetDlgItem(IDC_FIRSTROW)->GetWindowText(txt);
	GetDlgItem(IDC_FIRSTROW)->SetWindowText(m_who + txt);
	
	CenterWindow();
	return TRUE;
}





/////////////////////////////////////////////////////////////////////////////
// CTheApp

BOOL CTheApp::InitInstance()
{
	// Dark Gray BackGround
	SetDialogBkColor(RGB(0xc0,0xc0,0xc0));
	
	m_pMainWnd = new CMainWindow();
	m_pMainWnd->RedrawWindow(NULL,NULL,RDW_INVALIDATE|RDW_ERASE|RDW_FRAME|RDW_UPDATENOW);

	// Load .INI Data
    ((CMainWindow*)m_pMainWnd)->m_sound =
    		GetProfileInt(OPTIONS_SECTION,SOUND_ENTRY,DEF_SOUND);
    ((CMainWindow*)m_pMainWnd)->m_skill =
    		GetProfileInt(OPTIONS_SECTION,PC_SKILL_ENTRY,DEF_SKILL);

	// Start a new Game
	((CMainWindow*)m_pMainWnd)->StartGame();

    return TRUE;
}    





/////////////////////////////////////////////////////////////////////////////
// theApp:
// Just creating this application object runs the whole application.
//
CTheApp NEAR theApp;



