/////////////////////////////////////////////////////////////////////////////
// hip.h: application declaration file


#ifndef __HIP_H__
#define __HIP_H__

#include "afxext.h"

// Game Board Extents
#define HORZ_TILES		6
#define VERT_TILES		6
#define MAX_SQUARES		128	// Depends on HORZ_TILES*VERT_TILES

#define BORDER_WIDTH	16	// measured in pixels
#define BUTTON_WIDTH	42	// measured in pixels


// Status Bar Properties
#define STATUS_BAR_FONT_HEIGHT	8	
#define STATUS_BAR_FONT_NAME	"MS Sans Serif"


// PC Skills (% of blind moves)
#define LOW_SKILL		20
#define MED_SKILL		10
#define HIGH_SKILL		0


// Computer's Sleeping Time [ms]
// (Seems that PC is Thinking...)
#define THINKING_TIMEOUT	500
                          
                          
// .INI Data
#define OPTIONS_SECTION		"Options"
#define	SOUND_ENTRY			"Sound"
#define	PC_SKILL_ENTRY		"Skill"
#define DEF_SOUND			0
#define	DEF_SKILL			MED_SKILL


/////////////////////////////////////////////////////////////////////////////
// CSquare

class CSquare
{
public:
	CPoint	m_1st;
	CPoint	m_2nd;
	CPoint	m_3rd;
	CPoint	m_4th;

	void Traslate(int dx = 0, int dy = 0);

	BOOL operator==(CSquare square);   
	CSquare& operator=(CSquare square);
};
  
  
  
  
  
/////////////////////////////////////////////////////////////////////////////

class CMainWindow : public CFrameWnd
{
public:
	CMainWindow();       
	void StartGame();
	
protected:
	// Initialization
	void CreateFrame();
	void CreateTiles();
	void InitSquares();
	
	// Sound Generator
	void Beep(int frequency,int duration);
	void Sleep(clock_t wait);

	// Board Drawing
	void DrawFrame();
	void DrawTiles();
	
	// Tiles Placement
	void PlaceComputerTile();
	void PlaceHumanTile(int row, int col);
	
	// Ending Game Managers
	void EndGame(int looserTile);
	BOOL CheckForSquares(int* pWhoSquared = NULL);

public:
	// Public Data
	int		m_skill;
	BOOL	m_sound;                               

protected:
	// Tiles Families
	enum TILES 	{	EMPTY = 0,
					HUMAN,
					COMPUTER, };
				
	// Status Bar Strings
	CString		m_winStr;
	CString		m_lossStr;
	CString		m_drawStr;

	CString		m_waitingStr;
	CString		m_thinkingStr;

	CString		m_humanStr;
	CString		m_computerStr;
	CStatusBar	m_statusBar;
	
	// Other Protected Data
	BOOL			m_started;
	int				m_totSquares;
	CSquare			m_squares[MAX_SQUARES];
	int				m_gameTile[VERT_TILES][HORZ_TILES];
	CBitmapButton*	m_gameArea[VERT_TILES+2][HORZ_TILES+2];

	//{{AFX_MSG( CMainWindow )
	afx_msg void OnGameNew();
	afx_msg void OnGameExit();
	afx_msg void OnGameAbout();
	afx_msg void OnGameRules();
	afx_msg void OnGameSoundon();
	afx_msg void OnUpdateGameSoundon(CCmdUI* pCmdUI);
	afx_msg void OnDestroy();
	afx_msg void OnGameLevelHigh();
	afx_msg void OnUpdateGameLevelHigh(CCmdUI* pCmdUI);
	afx_msg void OnGameLevelLow();
	afx_msg void OnUpdateGameLevelLow(CCmdUI* pCmdUI);
	afx_msg void OnGameLevelMedium();
	afx_msg void OnUpdateGameLevelMedium(CCmdUI* pCmdUI);
	//}}AFX_MSG
	
	// Status Bar Panes Enabling
	afx_msg void UpdateHumanPane(CCmdUI* pCmdUI);
	afx_msg void UpdateComputerPane(CCmdUI* pCmdUI);

	virtual BOOL OnCommand(WPARAM wParam, LPARAM lParam);

	DECLARE_MESSAGE_MAP()
};





/////////////////////////////////////////////////////////////////////////////
// CStartDlg dialog

class CStartDlg : public CDialog
{
// Construction
public:
	CStartDlg(CWnd* pParent = NULL);	// standard constructor
	BOOL Create(CWnd* pParent = NULL);

// Dialog Data
	//{{AFX_DATA(CStartDlg)
	enum { IDD = IDD_STARTDLG };
		// NOTE: the ClassWizard will add data members here
	//}}AFX_DATA

	// Public Interface
	CString m_who;
	
// Implementation
protected:
	virtual void DoDataExchange(CDataExchange* pDX);	// DDX/DDV support

	// Generated message map functions
	//{{AFX_MSG(CStartDlg)
	virtual BOOL OnInitDialog();
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
};





/////////////////////////////////////////////////////////////////////////////
// CTheApp:
// See hello.cpp for the code to the InitInstance member function.
//
class CTheApp : public CWinApp
{
public:
	BOOL InitInstance();
};

/////////////////////////////////////////////////////////////////////////////

#endif // __HIP_H__

